@@ -38,7 +38,19 @@ class BlueprintableList(MutableSequence):
         self.data = []
         if initlist is not None:
             for elem in initlist:
-                self.append(elem)
+                if isinstance(elem, Blueprint):
+                    self.append(elem)
+                elif isinstance(elem, BlueprintBook):
+                    self.append(elem)
+                elif "blueprint" in elem:
+                    self.append(Blueprint(elem["blueprint"]))
+                elif "blueprint_book" in elem:
+                    self.append(BlueprintBook(elem["blueprint_book"]))
+                else:
+                    raise TypeError(
+                        "Element of blueprint book is not blueprint or \
+                        blueprint book object nor is it blueprint dictionary"
+                    )
 
     def insert(self, idx, value):
         # type: (int, Union[Blueprint, BlueprintBook]) -> None
